#include <Arduino.h>
#include <Adafruit_TinyUSB.h>
#include <SparkFunLIS3DH.h>

#define INT1 WB_IO5
#define INT2 WB_IO6

int eventType = 0;
bool freeFallFlag = 0;
bool motionFlag = 0;

/*Aporte Gabriel Parte Interrupciones */
void freeFallInterruptHandler()
{
	freeFallFlag = 1;
}

void wakeUpInterruptHandler()
{
	motionFlag = 1;
}

#include "SparkFunLIS3DH.h"
#include "Wire.h"

LIS3DH myIMU(I2C_MODE, 0x18); // Default constructor is I2C, addr 0x19.

void configIntteruptsAcelerometer()
{
	uint8_t dataToWrite = 0;

	// LIS3DH_INT1_CFG
	// dataToWrite |= 0x80;//AOI, 0 = OR 1 = AND
	// dataToWrite |= 0x40;//6D, 0 = interrupt source, 1 = 6 direction source
	// Set these to enable individual axes of generation source (or direction)
	//  -- high and low are used generically
	// dataToWrite |= 0x20;//Z high
	// dataToWrite |= 0x10;//Z low
	dataToWrite |= 0x08; // Y high
	// dataToWrite |= 0x04;//Y low
	// dataToWrite |= 0x02;//X high
	// dataToWrite |= 0x01;//X low
	myIMU.writeRegister(LIS3DH_INT1_CFG, dataToWrite);

	// LIS3DH_INT1_THS
	dataToWrite = 0;
	// Provide 7 bit value, 0x7F always equals max range by accelRange setting
	dataToWrite |= 0x10; // 1/8 range
	myIMU.writeRegister(LIS3DH_INT1_THS, dataToWrite);

	// LIS3DH_INT1_DURATION
	dataToWrite = 0;
	// minimum duration of the interrupt
	// LSB equals 1/(sample rate)
	dataToWrite |= 0x01; // 1 * 1/50 s = 20ms
	myIMU.writeRegister(LIS3DH_INT1_DURATION, dataToWrite);

	// LIS3DH_CLICK_CFG
	dataToWrite = 0;
	// Set these to enable individual axes of generation source (or direction)
	//  -- set = 1 to enable
	// dataToWrite |= 0x20;//Z double-click
	dataToWrite |= 0x10; // Z click
	// dataToWrite |= 0x08;//Y double-click
	dataToWrite |= 0x04; // Y click
	// dataToWrite |= 0x02;//X double-click
	dataToWrite |= 0x01; // X click
	myIMU.writeRegister(LIS3DH_CLICK_CFG, dataToWrite);

	// LIS3DH_CLICK_SRC
	dataToWrite = 0;
	// Set these to enable click behaviors (also read to check status)
	//  -- set = 1 to enable
	// dataToWrite |= 0x20;//Enable double clicks
	dataToWrite |= 0x04; // Enable single clicks
	// dataToWrite |= 0x08;//sine (0 is positive, 1 is negative)
	dataToWrite |= 0x04; // Z click detect enabled
	dataToWrite |= 0x02; // Y click detect enabled
	dataToWrite |= 0x01; // X click detect enabled
	myIMU.writeRegister(LIS3DH_CLICK_SRC, dataToWrite);

	// LIS3DH_CLICK_THS
	dataToWrite = 0;
	// This sets the threshold where the click detection process is activated.
	// Provide 7 bit value, 0x7F always equals max range by accelRange setting
	dataToWrite |= 0x0A; // ~1/16 range
	myIMU.writeRegister(LIS3DH_CLICK_THS, dataToWrite);

	// LIS3DH_TIME_LIMIT
	dataToWrite = 0;
	// Time acceleration has to fall below threshold for a valid click.
	// LSB equals 1/(sample rate)
	dataToWrite |= 0x08; // 8 * 1/50 s = 160ms
	myIMU.writeRegister(LIS3DH_TIME_LIMIT, dataToWrite);

	// LIS3DH_TIME_LATENCY
	dataToWrite = 0;
	// hold-off time before allowing detection after click event
	// LSB equals 1/(sample rate)
	dataToWrite |= 0x08; // 4 * 1/50 s = 160ms
	myIMU.writeRegister(LIS3DH_TIME_LATENCY, dataToWrite);

	// LIS3DH_TIME_WINDOW
	dataToWrite = 0;
	// hold-off time before allowing detection after click event
	// LSB equals 1/(sample rate)
	dataToWrite |= 0x10; // 16 * 1/50 s = 320ms
	myIMU.writeRegister(LIS3DH_TIME_WINDOW, dataToWrite);

	// LIS3DH_CTRL_REG5
	// Int1 latch interrupt and 4D on  int1 (preserve fifo en)
	myIMU.readRegister(&dataToWrite, LIS3DH_CTRL_REG5);
	dataToWrite &= 0xF3; // Clear bits of interest
	dataToWrite |= 0x08; // Latch interrupt (Cleared by reading int1_src)
	// dataToWrite |= 0x04; //Pipe 4D detection from 6D recognition to int1?
	myIMU.writeRegister(LIS3DH_CTRL_REG5, dataToWrite);

	// LIS3DH_CTRL_REG3
	// Choose source for pin 1
	dataToWrite = 0;
	// dataToWrite |= 0x80; //Click detect on pin 1
	dataToWrite |= 0x40; // AOI1 event (Generator 1 interrupt on pin 1)
	dataToWrite |= 0x20; // AOI2 event ()
	// dataToWrite |= 0x10; //Data ready
	// dataToWrite |= 0x04; //FIFO watermark
	// dataToWrite |= 0x02; //FIFO overrun
	myIMU.writeRegister(LIS3DH_CTRL_REG3, dataToWrite);

	// LIS3DH_CTRL_REG6
	// Choose source for pin 2 and both pin output inversion state
	dataToWrite = 0;
	dataToWrite |= 0x80; // Click int on pin 2
	// dataToWrite |= 0x40; //Generator 1 interrupt on pin 2
	// dataToWrite |= 0x10; //boot status on pin 2
	// dataToWrite |= 0x02; //invert both outputs
	myIMU.writeRegister(LIS3DH_CTRL_REG6, dataToWrite);

	//////////////////////////////////////////
	// // Acá empezamos config wake-up con filtro pasa altos y frefall

	// // Paso 1
	// uint8_t dataToWrite = 0;
	// dataToWrite |= 0x57; // WakeUP INT Config: Encendemos el sensor y activamos ejes X, Y, Z y activamos ODR (sample rate) a 100hz
	// dataToWrite |= 0x57; // FreeFall INT Config: (01010111 binario) Encendemos el sensor y activamos ejes X, Y, Z y activamos ODR (sample rate) a 100hz
	// myIMU.writeRegister(LIS3DH_CTRL_REG1, dataToWrite);

	// // Paso 2
	// dataToWrite = 0;
	// dataToWrite |= 0xA;	 // WakeUP INT Config: Filtro pasa altos activado sobre la actividad de interrupción 2 (on interrupt activity 2)
	// dataToWrite |= 0x00; // FreeFall INT Config: Filtro pasa altos desactivado
	// myIMU.writeRegister(LIS3DH_CTRL_REG2, dataToWrite);

	// // Paso 3
	// dataToWrite = 0;
	// dataToWrite |= 0x40; // FreeFall INT Config: Enlazamos Interrupción 1 al pin INT1 del sensor
	// myIMU.writeRegister(LIS3DH_CTRL_REG3, dataToWrite);

	// // Paso 3
	// dataToWrite = 0;
	// dataToWrite |= 0x20; // WakeUP INT Config: Enlazamos Actividad de Interrupción 2 al pin INT2 del sensor
	// myIMU.writeRegister(LIS3DH_CTRL_REG6, dataToWrite);

	// // Paso 4
	// dataToWrite = 0;
	// dataToWrite |= 0x00; // WakeUP INT Config: FS = +-2g
	// dataToWrite |= 0x00; // FreeFall INT Config: FS = +-2g
	// myIMU.writeRegister(LIS3DH_CTRL_REG4, dataToWrite);

	// // Paso 5
	// dataToWrite = 0;
	// dataToWrite |= 0x02; // WakeUP INT Config: Convertimos el pin INT2 del sensor en latch (interruptor)
	// dataToWrite |= 0x08; // FreeFall INT Config: Convertimos el pin INT1 del sensor en latch (interruptor)
	// myIMU.writeRegister(LIS3DH_CTRL_REG5, dataToWrite);

	// // Paso 6
	// dataToWrite = 0;
	// dataToWrite |= 0x10;					// WakeUP INT Config: Configuramos umbral de caída libre en 250mg (0x10 es 16 decimal. Con 2g, 1 lsb es 16mg según datasheet, entonces 16 decimal * 16mg = 256mg)
	// myIMU.writeRegister(0x36, dataToWrite); // 0x36 = INT2_THS

	// // Paso 7
	// dataToWrite = 0;
	// dataToWrite |= 0xA;						// WakeUP INT Config:  0.1 segundos
	// myIMU.writeRegister(0x37, dataToWrite); // 0x37 = INT2_DURATION

	// // Paso 8
	// uint8_t dataRead;
	// myIMU.readRegister(&dataRead, LIS3DH_REFERENCE); // WakeUP INT Config: leemos el registro REFERENCE

	// // Paso 9
	// dataToWrite = 0;
	// dataToWrite |= 0x2A;					// Configuramos el evento de wake-up deseado
	// myIMU.writeRegister(0x34, dataToWrite); // WakeUP INT Config: 0x34 = INT2_CFG

	// // Acá terminamos config wake-up

	// // Acá seguimos configurando freefall
	// // Paso 6
	// dataToWrite = 0;
	// dataToWrite |= 0x16; // FreeFall INT Config: Configuramos umbral de caída libre en 350mg
	// myIMU.writeRegister(LIS3DH_INT1_THS, dataToWrite);

	// // Paso 7
	// dataToWrite = 0;
	// dataToWrite |= 0x1E; // FreeFall INT Config: 0.3 segundos
	// myIMU.writeRegister(LIS3DH_INT1_DURATION, dataToWrite);

	// // Paso 8
	// dataToWrite = 0;
	// dataToWrite |= 0x95; // FreeFall INT Config: Configuramos la detección de caída libre
	// myIMU.writeRegister(LIS3DH_INT1_CFG, dataToWrite);

	// // Acá terminamos config freefall
}

void setupAcelerometer()
{

	myIMU.begin();

	configIntteruptsAcelerometer();
}

void clearAcelerometerRegistersWakeUp()
{

	uint8_t dataRead;
	myIMU.readRegister(&dataRead, 0x35); // cleared by reading, 0x35 = LIS3DH_INT2_SRC
}

void clearAcelerometerRegistersFreeFall()
{

	uint8_t dataRead;
	myIMU.readRegister(&dataRead, LIS3DH_INT1_SRC); // cleared by reading
}

void setup()
{

	pinMode(INT1, INPUT);
	pinMode(INT2, INPUT);
	pinMode(LED_BLUE, OUTPUT);
	pinMode(LED_GREEN, OUTPUT);
	digitalWrite(LED_BLUE, LOW);
	digitalWrite(LED_GREEN, LOW);

	// put your setup code here, to run once:
	Serial.begin(115200);
	delay(1000); // relax...

	setupAcelerometer();

	// setup DIO interrupt FreeFall
	pinMode(INT1, INPUT);
	attachInterrupt(digitalPinToInterrupt(INT1), freeFallInterruptHandler, RISING);

	// setup DIO interrupt wakeup
	pinMode(INT2, INPUT);
	attachInterrupt(digitalPinToInterrupt(INT2), wakeUpInterruptHandler, RISING);
}

void loop()
{

	if (freeFallFlag)
	{
		Serial.println("INT1 activada (FreeFall)");
		clearAcelerometerRegistersFreeFall();
		freeFallFlag = 0;
		digitalToggle(LED_BLUE);
	}

	if (motionFlag)
	{

		if (freeFallFlag)
		{

			clearAcelerometerRegistersWakeUp();
			Serial.println("INT1 activada (FreeFall) y también wakeup int");
			freeFallFlag = 0;
			motionFlag = 0;
			digitalToggle(LED_BLUE);
		}
		else
		{
			Serial.println("INT2 activada (Motion)");
			clearAcelerometerRegistersWakeUp();
			motionFlag = 0;
			digitalToggle(LED_GREEN);
		}
	}

	Serial.println("########################");

	delay(1000);
}